#!/usr/bin/python
# -*- coding:utf-8 -*- 

import psycopg2
import sys, getopt
import pandas
import csv
#import statsmodels.formula.api as smf
from sklearn import preprocessing
import math
import time
from heapq import *
import re
import itertools
import bisect

from capexplain.similarity.category_similarity_matrix import *
from capexplain.similarity.category_network_embedding import *
from capexplain.utils import *
from capexplain.pattern_model.LocalRegressionPattern import *


DEFAULT_QUESTION_PATH = './input/user_question_crime.csv'

DEFAULT_QUERY_RESULT_TABLE = 'crime_2017_2'
# DEFAULT_PATTERN_TABLE = 'pub_select'
DEFAULT_PATTERN_TABLE = 'crime_2017_2'
DEFAULT_USER_QUESTION_NUMBER = 5
EXAMPLE_NETWORK_EMBEDDING_PATH = './input/NETWORK_EMBEDDING'
EXAMPLE_SIMILARITY_MATRIX_PATH = './input/SIMILARITY_DEFINITION'
DEFAULT_AGGREGATE_COLUMN = '*'
DEFAULT_EPSILON = 0.25
TOP_K = 5

def build_local_regression_constraint(data, column_index, t, con, epsilon, agg_col, regression_package):
    """Build local regression constraint from Q(R), t, and global regression constraint

    Args:
        data: result of Q(R)
        column_index: index for values in each column
        t: target tuple in Q(R)
        con: con[0] is the list of fixed attributes in Q(R), con[1] is the list of variable attributes in Q(R)
        epsilon: threshold for local regression constraint
        regression_package: which package is used to compute regression 
    Returns:
        A LocalRegressionConstraint object whose model is trained on \pi_{con[1]}(Q_{t[con[0]]}(R))
    """
    tF = get_F_value(con[0], t)
    local_con = LocalRegressionConstraint(con[0], tF, con[1], agg_col, epsilon)
    train_data = {agg_col: []}
    for v in con[1]:
        train_data[v] = []
    # for index, row in data['df'].iterrows():
    #     if get_F_value(con[0], row) == tF:
    #         for v in con[1]:
    #             train_data[v].append(row[v])
    #         train_data[agg_col].append(row[agg_col])

    
    for idx in column_index[con[0][0]][tF[0]]:
        row = data['df'].loc[data['df']['index'] == idx]
        row = row.to_dict('records')[0]
        #print row
        if get_F_value(con[0], row) == tF:
            for v in con[1]:
                train_data[v].append(row[v])
            train_data[agg_col].append(row[agg_col])
    if regression_package == 'scikit-learn':
        train_x = {}
        for v in con[1]:
            if v in data['le']:
                train_data[v] = data['le'][v].transform(train_data[v])
                train_data[v] = data['ohe'][v].transform(train_data[v].reshape(-1, 1))
                #print data['ohe'][v].transform(train_data[v].reshape(-1, 1))
                train_x[v] = train_data[v]
            else:
                if v != agg_col:
                    train_x[v] = np.array(train_data[v]).reshape(-1, 1)
        train_y = np.array(train_data[agg_col]).reshape(-1, 1)
        train_x = np.concatenate(list(train_x.values()), axis=-1)
        local_con.train_sklearn(train_x, train_y)
    else:
        #train_data = pandas.DataFrame(train_data)
        formula = agg_col + ' ~ ' + ' + '.join(con[1])
        local_con.train(train_data, formula)
    return local_con

        
def tuple_similarity(t1, t2, var_attr, cat_sim, num_dis_norm, agg_col):
    """Compute the similarity between two tuples t1 and t2 on their attributes var_attr

    Args:
        t1, t2: two tuples
        var_attr: variable attributes
        cat_sim: the similarity measure for categorical attributes
        num_dis_norm: normalization terms for numerical attributes
        agg_col: the column of aggregated value
    Returns:
        the Gower similarity between t1 and t2
    """
    sim = 0.0
    cnt = 0
    for v_col in var_attr:
        col = v_col.replace(' ', '')
        
        if t1[col] is None or t2[col] is None:
            continue
        if cat_sim.is_categorical(col):
            t1_key = t1[col].replace("'", '').replace(' ', '')
            t2_key = t2[col].replace("'", '').replace(' ', '')
            s = cat_sim.compute_similarity(col, t1_key, t2_key, agg_col)
            # print(s)
            sim += s
        else:
            # print( num_dis_norm[col]['range'])
            if num_dis_norm[col]['range'] is None:
                if t1[col] == t2[col]:
                    sim += 1
            else:
                if col != agg_col and col != 'index':
                    temp = abs(t1[col] - t2[col]) / num_dis_norm[col]['range']
                    sim += 1-temp
        cnt += 1
    # print(t1, t2, sim)
    return sim / cnt

def get_local_patterns(global_patterns, t, cur, table_name):
    local_patterns = []
    local_patterns_dict = {}
    for pat in global_patterns:
        tF = get_F_value(pat[0], t)
        print(pat)
        local_pattern_query = '''SELECT * FROM {} WHERE REPLACE(fixed_value, ' ', '')=REPLACE('{}', ' ', '') AND variable='{}' AND in_a='{}' AND agg='{}' AND model='{}';'''.format(
                table_name + '_local', str(tF).replace("\'", '"'), str(pat[1]).replace("\'", ''), pat[2], pat[3], pat[4]
            )
        # local_pattern_query = '''SELECT * FROM {} WHERE REPLACE(fixed_value, ' ', '')=REPLACE('{}', ' ', '') AND variable='{}' AND in_a='{}' AND agg='{}' AND model='{}';'''.format(
        #           table_name + '_local', str(tF).replace("\'", ''), str(pat[1]).replace("\'", ''), pat[2], pat[3], pat[4]
        # )
        print(local_pattern_query)
        cur.execute(local_pattern_query)
        res = cur.fetchall()
        l_pat = []
        print('In get local patterns: ', res)
        if len(res) != 0:
            local_patterns_dict[str(res[0])] = True
            for i in range(3):
                l_pat.append(res[0][i][1:-1].split(','))
            l_pat[0] = list(map(lambda x: x.replace(' ', ''), l_pat[0]))
            l_pat[2] = list(map(lambda x: x.replace(' ', ''), l_pat[2]))
            for i in range(3, len(res[0])):
                l_pat.append(res[0][i])
            # print(l_pat)
            local_patterns.append(l_pat)
        else:
            l_pat.append(pat[0])
            l_pat.append(tF)
            for i in range(1, len(pat)):
                l_pat.append(pat[i])
            local_patterns.append({'hold': False, 'pattern': l_pat})


    for num_of_fix in range(1, min(len(t.keys()),4)):
        for possible_f_attr in itertools.combinations(list(t.keys())[:-1], num_of_fix):
            tF = get_F_value(possible_f_attr, t)
            local_pattern_query = '''SELECT * FROM {} WHERE REPLACE(fixed_value, ' ', '')=REPLACE('{}', ' ', '')'''.format(
                table_name + '_local', str(tF).replace("\'", '"'))
            # print(tF)
            print(local_pattern_query)
            cur.execute(local_pattern_query)
            res = cur.fetchall()
            # print('In get local patterns: ', res)
            for k in range(len(res)):
                l_pat = []
                if str(res[k]) in local_patterns_dict:
                    continue
                else:
                    local_patterns_dict[str(res[k])] = True
                for i in range(3):
                    l_pat.append(res[k][i][1:-1].split(','))
                l_pat[0] = list(map(lambda x: x.replace(' ', ''), l_pat[0]))
                l_pat[2] = list(map(lambda x: x.replace(' ', ''), l_pat[2]))
                for i in range(3, len(res[0])):
                    l_pat.append(res[0][i])
                # print(l_pat)
                if l_pat not in local_patterns:
                    local_patterns.append(l_pat)
    print('GET ', local_patterns)
    return local_patterns


def get_tuples_by_F(local_pattern, f_value, cur, table_name):
    
    where_clause = ' AND '.join(list(map(lambda x: "{} {}".format(x[0], x[1]), zip(local_pattern[0], map(tuple_column_to_str_in_where_clause, f_value)))))
    tuples_query = "SELECT * FROM {} WHERE {};".format(table_name, where_clause)
    
    column_name_query = "SELECT column_name FROM information_schema.columns where table_name='{}';".format(table_name)
    # print(column_name_query)
    cur.execute(column_name_query)
    column_name = cur.fetchall()
    cur.execute(tuples_query)
    # print(tuples_query)
    tuples = []
    res = cur.fetchall()
    for row in res:
        tuples.append(dict(zip(map(lambda x: x[0], column_name), row)))
    return tuples


def load_user_question(uq_path=DEFAULT_QUESTION_PATH):
    '''
        load user questions
    '''
    uq = []
    with open(uq_path, 'rt') as uqfile:
        reader = csv.DictReader(uqfile, quotechar='\'')
        headers = reader.fieldnames
        #temp_data = csv.reader(uqfile, delimiter=',', quotechar='\'')
        #for row in temp_data:
        for row in reader:
            row_data = {}
            for k, v in enumerate(headers):
                # print(k, v)
                if v != 'direction':
                    if is_float(row[v]):
                        row_data[v] = float(row[v])
                    elif is_integer(row[v]):
                        row_data[v] = float(long(row[v]))
                    else:
                        row_data[v] = row[v]
            if row['direction'][0] == 'h':
                dir = 1
            else:
                dir = -1
            uq.append({'target_tuple': row_data, 'dir':dir})
    return uq

def load_patterns(cur, pat_table_name):
    '''
        load pre-defined constraints(currently only fixed attributes and variable attributes)
    '''
    global_pattern_table = pat_table_name + '_global'
    load_query = "SELECT * FROM {};".format(global_pattern_table)
    cur.execute(load_query)
    res = cur.fetchall()
    patterns = []
    for pat in res:
        patterns.append(list(pat))
        patterns[-1][0] = patterns[-1][0][1:-1].replace(' ', '').split(',')
        patterns[-1][1] = patterns[-1][1][1:-1].replace(' ', '').split(',')
    
    return patterns


def get_local_patterns(global_patterns, t, cur, table_name):
    local_patterns = []
    local_patterns_dict = {}
    for pat in global_patterns:
        
        F_set = set(pat[0])
        V_set = set(pat[1])
        t_set = set(t.keys())
        FV_set = F_set.union(V_set)
        if not (FV_set.issubset(t_set) and len(FV_set) == len(t_set) - 1 
            and '{}({})'.format(pat[3], pat[2]) in t):
        # if F_set.union(V_set) != t_set:
            continue
        # print(pat[0], t)
        local_patterns.append(get_local_pattern(pat, t, cur, table_name))

        
def count_gloabl_patterns(global_patterns, t, cur, table_name):
    local_patterns = []
    local_patterns_dict = {}
    g_pat_cnt_by_theta = [0 for i in range(21)]
    for pat in global_patterns:
        
        F_set = set(pat[0])
        V_set = set(pat[1])
        t_set = set(t.keys())
        FV_set = F_set.union(V_set)
        # print(pat, t)
        # print(FV_set, t_set, pat[3], pat[2])
        if not (FV_set.issubset(t_set) and len(FV_set) == len(t_set) - 2
            and '{}({})'.format(pat[3], pat[2]) in t):
        # if F_set.union(V_set) != t_set:
            continue
        # print(pat[0], t)
        # if float(pat[-1]) >= t['lambda']:
        local_pattern_query = '''SELECT theta FROM {} 
            WHERE fixed='{}' AND variable='{}' AND 
                in_a='{}' AND agg='{}' AND model='{}'
            ORDER BY theta;
        '''.format(
            table_name + '_local', str(pat[0]).replace("\'", ''), str(pat[1]).replace("\'", ''), pat[2], pat[3], pat[4] 
        )

        cur.execute(local_pattern_query)
        res = list(map(lambda x:x[0], cur.fetchall()))
        # print(res)
        for the_i in range(2, 21):
            j = bisect.bisect_left(res, the_i * 0.05)
            if len(res) - j >= t['lambda'] * len(res):
                g_pat_cnt_by_theta[the_i] += 1 
    print(g_pat_cnt_by_theta)

def main(argv=[]):
    query_result_table = DEFAULT_QUERY_RESULT_TABLE
    pattern_table = DEFAULT_PATTERN_TABLE
    user_question_file = DEFAULT_QUESTION_PATH
    outputfile = ''
    epsilon = DEFAULT_EPSILON
    aggregate_column = DEFAULT_AGGREGATE_COLUMN
    try:
        # conn = psycopg2.connect("host=216.47.152.61 port=5432 dbname=postgres user=antiprov password=test")
        # conn = psycopg2.connect("host=localhost port=5432 dbname=antiprov")
        conn = psycopg2.connect("host=localhost port=5432 dbname=antiprov user=zjmiao password=keertijeff")
        cur = conn.cursor()
    except psycopg2.OperationalError:
        print('数据库连接失败！')

    try:
        opts, args = getopt.getopt(argv,"hq:p:u:o:e:a",["qtable=", "ptable=", "ufile=","ofile=","epsilon=","aggregate_column="])
    except getopt.GetoptError:
        print('explanation.py -q <query_result_table> -p <pattern_table> -u <user_question_file> -o <outputfile> -e <epsilon> -a <aggregate_column>')
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-h':
            print('explanation.py -q <query_result_table> -p <pattern_table> -u <user_question_file> -o <outputfile> -e <epsilon> -a <aggregate_column>')
            sys.exit(2)    
        elif opt in ("-q", "--qtable"):
            query_result_table = arg
        elif opt in ("-p", "--ptable"):
            pattern_table = arg
        elif opt in ("-u", "--ufile"):
            user_question_file = arg
        elif opt in ("-o", "--ofile"):
            outputfile = arg
        elif opt in ("-e", "--epsilon"):
            epsilon = float(arg)
        elif opt in ("-a", "--aggcolumn"):
            aggregate_column = arg

    start = time.clock()
    global_patterns = load_patterns(cur, pattern_table)
    end = time.clock()
    print('Loading time: ' + str(end-start) + 'seconds')

    Q = load_user_question(user_question_file)
    print(global_patterns)
    print(Q)
    for uq in Q:
        count_gloabl_patterns(global_patterns, uq['target_tuple'], cur, pattern_table)
    
    
 

if __name__ == "__main__":
    main(sys.argv[1:])





import tkinter as tk
from tkinter import *
from tkinter import ttk
import tkinter.messagebox
from tkinter import filedialog
from tkinter import font
import pandas as pd
import psycopg2
from pandastable import TableModel
from pandastable import PlotViewer
from pandastable import Table
import re
from capexplain.explain.explanation import ExplanationGenerator
from capexplain.explain.explanation import ExplConfig
import matplotlib
matplotlib.use("TkAgg")
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg,NavigationToolbar2Tk
from matplotlib.figure import Figure
import matplotlib.pyplot as plt
import numpy as np
from mpl_toolkits.mplot3d import Axes3D
from capexplain.gui.Plotting import Plotter
import logging
import textwrap
import copy

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
formatter = logging.Formatter('%(asctime)s %(levelname)s %(name)s line %(lineno)d: %(message)s')
stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
logger.addHandler(stream_handler)



class Exp_Frame:

	def __init__(self,input_question_df=None, input_explanation_df=None, input_exp_chosen_row=None, input_none_drill_down_df=None, input_drill_down_df=None, input_data_convert_dict=None):

		self.win = Toplevel()
		self.win.geometry("%dx%d%+d%+d" % (1580, 900, 250, 125))
		self.win.wm_title("Explanation Detail")		
		self.win_frame = Frame(self.win)
		self.win_frame.pack(fill=BOTH,expand=True)

		self.question_df = input_question_df
		self.explanation_df = input_explanation_df
		self.exp_chosen_row = input_exp_chosen_row
		self.none_drill_down_df = input_none_drill_down_df

		# print("self.question_df type-----------------")
		# print(self.question_df.dtypes)

		# print("self.explanation_df type-----------------")
		# print(self.explanation_df.dtypes)

		# print("self.exp_chosen_row type-----------------")
		# print(self.exp_chosen_row.dtypes)

		# print("self.none_drill_down_df type-----------------")
		# print(self.none_drill_down_df.dtypes)


		if(input_drill_down_df is not None):
			self.drill_down_df = input_drill_down_df.astype(object)
			# print("self.drill_down_df type-----------------")
			# print(self.drill_down_df.dtypes)
		else:
			self.drill_down_df = None
		self.data_convert_dict=input_data_convert_dict
		self.drill_exist=False

		self.relevent_pattern = self.exp_chosen_row['From_Pattern']
		self.rel_pattern_part = self.relevent_pattern.split(':')[0].split('=')[0].strip('[')
		self.rel_pattern_pred = self.relevent_pattern.split(':')[1].split(' \u2933 ')[0]
		self.rel_pattern_agg = self.relevent_pattern.split(':')[1].split(' \u2933 ')[1]
		self.rel_pattern_part_value = self.relevent_pattern.split(':')[0].split('=')[1].strip(']')
		self.rel_pattern_pred_list = self.rel_pattern_pred.split(',')
		self.rel_pattern_model = self.exp_chosen_row['relevent_model']
		self.rel_param = self.exp_chosen_row['relevent_param']
		self.rel_pattern_part_list = self.rel_pattern_part.split(',')
		self.rel_pattern_pred_list = self.rel_pattern_pred.split(',')
		self.exp_tuple_score = float(self.exp_chosen_row['Score'])
		self.drill_attr = [self.exp_chosen_row['Drill_Down_To'].split(',')]
		self.drill_model = self.exp_chosen_row['refinement_model']
		self.drill_param = self.exp_chosen_row['drill_param']


	# configure the frame structure according the exp type

		if(self.drill_down_df is None):

			self.win_frame.columnconfigure(0,weight=2)
			self.win_frame.columnconfigure(1,weight=3)
			self.win_frame.rowconfigure(0,weight=8)
			self.win_frame.rowconfigure(1,weight=1)

			self.Quit_Button = Button(self.win_frame, text="Quit",width=10, height=4, command=self.win.destroy)
			self.Quit_Button.grid(column=0,row=1)

			self.rel_graph_frame = Frame(self.win_frame,borderwidth=5,relief=RIDGE,bg='white')
			self.rel_graph_frame.grid(column=1,row=0,rowspan=2,sticky='nesw')

			self.exp_frame = Frame(self.win_frame,borderwidth=5,relief=RIDGE,bg='white')
			self.exp_frame.grid(column=0,row=0,sticky='nesw')

			self.rel_figure = Figure(figsize=(5,5),dpi=130)

			self.rel_canvas = FigureCanvasTkAgg(self.rel_figure,self.rel_graph_frame)
			self.rel_canvas.get_tk_widget().pack(side=TOP, fill=BOTH, expand=True)
			self.rel_toolbar = NavigationToolbar2Tk(self.rel_canvas,self.rel_graph_frame)
			self.rel_toolbar.update()
			self.rel_canvas.get_tk_widget().pack(side=TOP, fill=BOTH, expand=1)
		
		else:

			self.drill_exist = True
			self.win_frame.columnconfigure(0,weight=1)
			self.win_frame.columnconfigure(1,weight=1)
			self.win_frame.rowconfigure(0,weight=2)
			self.win_frame.rowconfigure(1,weight=1)

			self.rel_graph_frame = Frame(self.win_frame,borderwidth=5,relief=RIDGE,bg='white')
			self.rel_graph_frame.grid(column=0,row=0,sticky='nesw')

			self.drill_graph_frame = Frame(self.win_frame,borderwidth=5,relief=RIDGE,bg='white')
			self.drill_graph_frame.grid(column=1,row=0,sticky='nesw')

			self.rel_figure = Figure(figsize=(5,5),dpi=130)
			self.rel_canvas = FigureCanvasTkAgg(self.rel_figure,self.rel_graph_frame)
			self.rel_canvas.get_tk_widget().pack(side=TOP, fill=BOTH, expand=True)
			self.rel_toolbar = NavigationToolbar2Tk(self.rel_canvas,self.rel_graph_frame)
			self.rel_toolbar.update()
			self.rel_canvas.get_tk_widget().pack(side=TOP, fill=BOTH, expand=1)

			self.drill_figure = Figure(figsize=(5,5),dpi=130)
			self.drill_canvas = FigureCanvasTkAgg(self.drill_figure,self.drill_graph_frame)
			self.drill_canvas.get_tk_widget().pack(side=TOP, fill=BOTH, expand=True)
			self.drill_toolbar = NavigationToolbar2Tk(self.drill_canvas,self.drill_graph_frame)
			self.drill_toolbar.update()
			self.drill_canvas.get_tk_widget().pack(side=TOP, fill=BOTH, expand=1)

			self.exp_frame = Frame(self.win_frame,borderwidth=5,relief=RIDGE,bg='white')
			self.exp_frame.grid(column=0,columnspan=2,row=1,sticky='nesw')


	def load_exp_graph(self):

		if(self.drill_exist==False):
			self.load_rel_exp_graph()
		else:
			self.load_rel_question_graph()
			self.load_drill_exp_graph()



	def load_rel_exp_graph(self):

		if(len(self.none_drill_down_df)>=50):

			self.text_plotter = Plotter(figure=self.rel_figure,data_convert_dict=self.data_convert_dict,mode='2D')
			self.rel_plotter.add_text("Cannot plot because the size of the data is so large!")


		elif(len(self.rel_pattern_pred_list)>2):

			self.text_plotter = Plotter(figure=self.rel_figure,data_convert_dict=self.data_convert_dict,mode='2D')
			self.rel_plotter.add_text("Cannot plot because the number of dimension of data is higher than 2!")


		elif(self.rel_pattern_model=='const'):

			if(len(self.rel_pattern_pred_list)==1):
				self.rel_plotter = Plotter(figure=self.rel_figure,data_convert_dict=self.data_convert_dict,mode='2D')
				const=round(float(self.rel_param),2)

				x=self.rel_pattern_pred_list[0]
				y=self.rel_pattern_agg

				self.rel_plotter.plot_2D_const(const,label='Explanation Model')
				# self.rel_plotter.plot_categorical_scatter_2D(x,y)
				none_drill_down_df = copy.deepcopy(self.none_drill_down_df[[x,y]])
				# logger.debug(none_drill_down_df)

				common_cols = self.rel_pattern_part_list + self.rel_pattern_pred_list

				# logger.debug("common_cols for question is ")
				# print(common_cols)

				# logger.debug("self.explanation_df is")
				# logger.debug(self.explanation_df)

				question_df = pd.merge(self.none_drill_down_df,self.question_df,on=common_cols)

				explanation_df = pd.merge(self.none_drill_down_df,self.explanation_df,on=common_cols)

				# logger.debug("question_df is ")
				# print(question_df)


				question_df = question_df.rename(index=str, columns={(y+"_x"): y,(x+"_x"):x})
				question_df = question_df[[x,y]]

				explanation_df = explanation_df.rename(index=str, columns={(y+"_x"): y,(x+"_x"):x})
				explanation_df = explanation_df[[x,y]]

				self.rel_plotter.plot_2D_scatter(question_df,x=x,y=y,color='r',marker='v',size=250,zorder=10,label="User Question")
				self.rel_plotter.plot_2D_scatter(explanation_df,x=x,y=y,color='b',marker='^',size=250,zorder=5,label="Explanation")
				self.rel_plotter.plot_2D_scatter(none_drill_down_df,x=x,y=y,zorder=0,label=self.rel_pattern_agg)
				self.rel_plotter.set_x_label(x)
				self.rel_plotter.set_y_label(y)
				self.rel_plotter.set_title("Pattern Graph")
		
			else:

				self.rel_plotter = Plotter(figure=self.rel_figure,data_convert_dict=self.data_convert_dict,mode='3D')

				x = self.rel_pattern_pred_list[0]
				y = self.rel_pattern_pred_list[1]
				z = self.rel_pattern_agg

				const = round(self.rel_param,2)

				none_drill_down_df = copy.deepcopy(self.none_drill_down_df[[x,y,z]])
				# logger.debug(none_drill_down_df)

				common_cols = self.rel_pattern_part_list + self.rel_pattern_pred_list

				# logger.debug("common_cols for question is ")
				# print(common_cols)

				# logger.debug("self.explanation_df is")
				# logger.debug(self.explanation_df)

				question_df = pd.merge(self.none_drill_down_df,self.question_df,on=common_cols)

				explanation_df = pd.merge(self.none_drill_down_df,self.explanation_df,on=common_cols)

				# logger.debug("question_df is ")
				# print(question_df)

				question_df = question_df.rename(index=str, columns={(y+"_x"): y,(x+"_x"):x,(z+"_x"):z})
				question_df = question_df[[x,y,z]]

				explanation_df = explanation_df.rename(index=str, columns={(y+"_x"): y,(z+"_x"):z})
				explanation_df = explanation_df[[x,y,z]]


				pattern_only_df = pd.concat([none_drill_down_df,question_df,explanation_df]).drop_duplicates(keep=False)

				self.rel_plotter.plot_3D_const(none_drill_down_df,x=x,y=y,z_value=const,label="Explanation Model")
				self.rel_plotter.plot_3D_scatter(none_drill_down_df,x=x,y=y,z=z,alpha=0)
				self.rel_plotter.plot_3D_scatter(pattern_only_df,x=x,y=y,z=z,label=self.rel_pattern_agg)
				self.rel_plotter.plot_3D_scatter(question_df,x=x,y=y,z=z,color='r',marker='v',size=250,label="User Question")
				self.rel_plotter.plot_3D_scatter(explanation_df,x=x,y=y,z=z,color='b',marker='^',size=250,label="Explanation")

				self.rel_plotter.set_x_label(x)
				self.rel_plotter.set_y_label(y)
				self.rel_plotter.set_z_label(z)
				self.rel_plotter.set_title("Pattern Graph")

		elif(self.rel_pattern_model=='linear'):
			if(len(self.rel_pattern_pred_list)==1):

				self.rel_plotter = Plotter(figure=self.rel_figure,data_convert_dict=self.data_convert_dict,mode='2D')
				x = self.rel_pattern_pred_list[0]
				y = self.rel_pattern_agg

				intercept_value = self.rel_param['Intercept']
				slope_name = list(self.rel_param)[1]
				slope_value = float(self.rel_param[slope_name])

				draw_line_df = copy.deepcopy(self.none_drill_down_df[[x]])

				none_drill_down_df = copy.deepcopy(self.none_drill_down_df[[x,y]])

				common_cols = self.rel_pattern_part_list + self.rel_pattern_pred_list

				# logger.debug("common_cols for question is ")
				# print(common_cols)

				# logger.debug("self.explanation_df is")
				# logger.debug(self.explanation_df)

				question_df = pd.merge(self.none_drill_down_df,self.question_df,on=common_cols)

				explanation_df = pd.merge(self.none_drill_down_df,self.explanation_df,on=common_cols)

				# logger.debug("question_df is ")
				# print(question_df)


				question_df = question_df.rename(index=str, columns={(y+"_x"): y,(x+"_x"):x})
				question_df = question_df[[x,y]]

				explanation_df = explanation_df.rename(index=str, columns={(y+"_x"): y,(x+"_x"):x})
				explanation_df = explanation_df[[x,y]]

				self.rel_plotter.plot_2D_linear(draw_line_df,slope=slope_value,intercept=intercept_value,label="Explanation Model")
				self.rel_plotter.plot_2D_scatter(none_drill_down_df,x=x,y=y,label=self.rel_pattern_agg)
				self.rel_plotter.plot_2D_scatter(question_df,x=x,y=y,color='r',marker='v',size=250,zorder=1,label="User Question")
				self.rel_plotter.plot_2D_scatter(explanation_df,x=x,y=y,color='b',marker='^',size=250,zorder=2,label="Explanation")
				self.rel_plotter.set_x_label(x)
				self.rel_plotter.set_y_label(y)
				self.rel_plotter.set_title("Pattern Graph")

		self.rel_canvas.draw()


	def load_rel_question_graph(self):

		if(len(self.none_drill_down_df)>=50):

			self.text_plotter = Plotter(figure=self.drill_figure,data_convert_dict=self.data_convert_dict,mode='2D')
			self.rel_plotter.add_text("Cannot plot because the size of the data is so large!")


		elif(len(self.rel_pattern_pred_list)>2):
			
			self.text_plotter = Plotter(figure=self.drill_figure,data_convert_dict=self.data_convert_dict,mode='2D')
			self.rel_plotter.add_text("Cannot plot because the dimension of the data is higher than 2!")


		elif(self.rel_pattern_model=='const'):
			if(len(self.rel_pattern_pred_list)==1):
				self.rel_plotter = Plotter(figure=self.rel_figure,data_convert_dict=self.data_convert_dict,mode='2D')
				const=round(float(self.rel_param),2)

				x=self.rel_pattern_pred_list[0]
				y=self.rel_pattern_agg

				self.rel_plotter.plot_2D_const(const)
				# self.rel_plotter.plot_categorical_scatter_2D(x,y)

				common_cols = self.rel_pattern_part_list + self.rel_pattern_pred_list

				# logger.debug("common_cols for question is ")
				# print(common_cols)

				# logger.debug("self.explanation_df is")
				# logger.debug(self.explanation_df)

				question_df = pd.merge(self.none_drill_down_df,self.question_df,on=common_cols)

				explanation_df = pd.merge(self.none_drill_down_df,self.explanation_df,on=common_cols)

				# logger.debug("question_df is ")
				# print(question_df)


				question_df = question_df.rename(index=str, columns={(y+"_x"): y,(x+"_x"):x})
				question_df = question_df[[x,y]]

				explanation_df = explanation_df.rename(index=str, columns={(y+"_x"): y,(x+"_x"):x})
				explanation_df = explanation_df[[x,y]]

				# logger.debug(question_df)

				self.rel_plotter.plot_2D_scatter(question_df,x=x,y=y,color='r',marker='v',size=250,zorder=10,label="User Question")
				self.rel_plotter.plot_2D_scatter(copy.deepcopy(self.none_drill_down_df),x=x,y=y,zorder=0,label=self.rel_pattern_agg)
				self.rel_plotter.plot_2D_scatter(explanation_df,x=x,y=y,color='b',marker='^',size=250,zorder=0,label="Explanation")
				self.rel_plotter.set_x_label(x)
				self.rel_plotter.set_y_label(y)
				self.rel_plotter.set_title("User Question Graph")
		
			else:


				pass

				# self.rel_plotter = Plotter(figure=self.rel_figure,data_convert_dict=self.data_convert_dict,mode='3D')

				# x = self.rel_pattern_pred_list[0]
				# y = self.rel_pattern_pred_list[1]
				# z = self.rel_pattern_agg

				# const = round(float(self.rel_param),2)

				# none_drill_down_df = self.none_drill_down_df[[x,y,z]]
				# logger.debug(none_drill_down_df)

				# question_df = self.question_df[[x,y,z]]
				# logger.debug(question_df)


				# pattern_only_df = pd.concat([none_drill_down_df,question_df]).drop_duplicates(keep=False)

				# self.rel_plotter.plot_3D_const(none_drill_down_df,x=x,y=y,z_value=const)
				# self.rel_plotter.plot_3D_scatter(none_drill_down_df,x=x,y=y,z=z,alpha=0)
				# self.rel_plotter.plot_3D_scatter(pattern_only_df,x=x,y=y,z=z)
				# self.rel_plotter.plot_3D_scatter(question_df,x=x,y=y,z=z,color='b',marker='s',size=200)

				# self.rel_plotter.set_x_label(x)
				# self.rel_plotter.set_y_label(y)
				# self.rel_plotter.set_z_label(z)
				# self.rel_plotter.set_title("User Question Graph")

		elif(self.rel_pattern_model=='linear'):
			if(len(self.rel_pattern_pred_list)==1):

				self.rel_plotter = Plotter(figure=self.rel_figure,data_convert_dict=self.data_convert_dict,mode='2D')
				x = self.rel_pattern_pred_list[0]
				y = self.rel_pattern_agg

				intercept_value = self.rel_param['Intercept']
				slope_name = list(self.rel_param)[1]
				slope_value = float(self.rel_param[slope_name])

				draw_line_df = self.none_drill_down_df[[x]]

				common_cols = self.rel_pattern_part_list+ self.rel_pattern_pred_list
				question_df = pd.merge(self.none_drill_down_df,self.question_df,on=common_cols)
				logger.debug(self.none_drill_down_df)
				logger.debug(self.explanation_df)
				explanation_df = pd.merge(self.none_drill_down_df,self.explanation_df,on=common_cols)

				# logger.debug("question_df is ")
				# print(question_df)


				question_df = question_df.rename(index=str, columns={(y+"_x"): y,(x+"_x"):x})
				question_df = question_df[[x,y]]
				logger.debug(question_df)

				explanation_df = explanation_df.rename(index=str, columns={(y+"_x"): y,(x+"_x"):x})
				explanation_df = explanation_df[[x,y]]
				logger.debug(explanation_df)

				self.rel_plotter.plot_2D_linear(draw_line_df,slope=slope_value,intercept=intercept_value,label="Relevent Model")
				self.rel_plotter.plot_2D_scatter(copy.deepcopy(self.none_drill_down_df),x=x,y=y,label=self.rel_pattern_agg)
				self.rel_plotter.plot_2D_scatter(question_df,x=x,y=y,color='r',marker='v',size=150,zorder=1,label="User Question")
				logger.debug(explanation_df)
				self.rel_plotter.plot_2D_scatter(explanation_df,x=x,y=y,color='b',marker='^',size=150,zorder=0,label="Explanation")

				self.rel_plotter.set_x_label(x)
				self.rel_plotter.set_y_label(y)
				self.rel_plotter.set_title("User Question Graph")

		self.rel_canvas.draw()

	def load_drill_exp_graph(self):

		if(len(self.none_drill_down_df)>=50):
			self.drill_plotter.add_text("Cannot plot because the size of the data is so large!")

		elif(len(self.rel_pattern_pred_list)>2):
			self.drill_plotter.add_text("Cannot plot because the number of dimension of data is higher than 2!")

		if(self.drill_model=='const'):

			if(len(self.drill_attr)==1):
				self.drill_plotter = Plotter(figure=self.drill_figure,data_convert_dict=self.data_convert_dict,mode='2D')
				const=round(float(self.drill_param),2)

				x=self.rel_pattern_pred_list[0]
				y=self.rel_pattern_agg

				self.drill_plotter.plot_2D_const(const,label="Refined Explanation Model")
				# self.drill_plotter.plot_categorical_scatter_2D(x,y)

				common_cols = self.rel_pattern_part_list+ self.drill_attr+self.rel_pattern_pred_list


				self.drill_plotter.plot_2D_scatter(copy.deepcopy(self.explanation_df),x=x,y=y,color='b',marker='^',size=250,zorder=10,label="Explanation")
				logger.debug("After Drilldown explanation:")
				logger.debug(self.drill_plotter.x_max)
				logger.debug(self.drill_plotter.x_min)
				logger.debug(self.drill_plotter.y_max)
				logger.debug(self.drill_plotter.y_min)
				self.drill_plotter.plot_2D_scatter(copy.deepcopy(self.drill_down_df),x=x,y=y,zorder=0,label=self.rel_pattern_agg)
				logger.debug(copy.deepcopy(self.drill_down_df))
				logger.debug("After Drilldown scatters:")
				logger.debug(self.drill_plotter.x_max)
				logger.debug(self.drill_plotter.x_min)
				logger.debug(self.drill_plotter.y_max)
				logger.debug(self.drill_plotter.y_min)
				self.drill_plotter.set_x_label(x)
				self.drill_plotter.set_y_label(y)
				self.drill_plotter.set_title("Refined Pattern Explanation")
		
			else:
				pass

				# self.drill_plotter = Plotter(figure=self.drill_figure,data_convert_dict=self.data_convert_dict,mode='3D')

				# x = self.rel_pattern_pred_list[0]
				# y = self.rel_pattern_pred_list[1]
				# z = self.rel_pattern_agg

				# const = round(float(self.drill_param),2)

				# drill_down_df = self.drill_down_df[[x,y,z]]
				# logger.debug(drill_down_df)

				# explanation_df = self.explanation_df[[x,y,z]]
				# logger.debug(explanation_df)


				# pattern_only_df = pd.concat([drill_down_df,explanation_df]).drop_duplicates(keep=False)

				# self.drill_plotter.plot_3D_const(drill_down_df,x=x,y=y,z_value=const)
				# self.drill_plotter.plot_3D_scatter(drill_down_df,x=x,y=y,z=z,alpha=0)
				# self.drill_plotter.plot_3D_scatter(pattern_only_df,x=x,y=y,z=z)
				# self.drill_plotter.plot_3D_scatter(explanation_df,x=x,y=y,z=z,color='b',marker='s',size=200,label='Explanation')

				# self.drill_plotter.set_x_label(x)
				# self.drill_plotter.set_y_label(y)
				# self.drill_plotter.set_z_label(z)
				# self.drill_plotter.set_title("Pattern Graph")

				# self.drill_canvas.draw()


		elif(self.drill_model=='linear'):

			if(len(self.drill_attr)==1):
				self.drill_plotter = Plotter(figure=self.drill_figure,data_convert_dict=self.data_convert_dict,mode='2D')
				x = self.drill_attr[0]
				y = self.rel_pattern_agg

				intercept_value = self.drill_param['Intercept']
				slope_name = list(self.drill_param)[1]
				slope_value = float(self.drill_param[slope_name])

				draw_line_df = self.none_drill_down_df[[x]]

				common_cols = self.rel_pattern_part_list+self.drill_attr

				explanation_df = pd.merge(self.drill_down_df,self.explanation_df,on=common_cols)

				explanation_df = explanation_df.rename(index=str, columns={(y+"_x"): y,(x+"_x"):x})

				explanation_df = explanation_df[[x,y]]

				# logger.debug(explanation_df)
				self.drill_plotter.plot_2D_linear(draw_line_df,slope=slope_value,intercept=intercept_value)
				self.drill_plotter.plot_2D_scatter(copy.deepcopy(self.drill_down_df)	,x=x,y=y)
				self.drill_plotter.plot_2D_scatter(explanation_df,x=x,y=y,zorder=1)
				self.drill_plotter.set_x_label(x)
				self.drill_plotter.set_y_label(y)
				self.drill_plotter.set_title("Pattern Graph")

		self.drill_canvas.draw()

	def load_exp_description(self,user_direction=None):

		exp_tuple_score = float(self.exp_chosen_row['Score'])

		likelihood_words = []

		if(exp_tuple_score<=0):
			likelihood_words = ['unlikely','not similar','slighlty']
		elif(exp_tuple_score<=10):
			likelihood_words = ['plausible','somewhat similar','']
		else:
			likelihood_words = ['highly plausible','similar','extremly']

		ranking_clause = "  This explanation was ranked "+likelihood_words[0] + " because the counterbalance is " + likelihood_words[1]+" to the user question and it deviates "+likelihood_words[2]+" from the predicted outcome."

		# logger.debug('ranking_clause:')
		# logger.debug(ranking_clause)

		# logger.debug('self.question_df is:')
		# logger.debug(self.question_df)

		user_question_list = []
		# logger.debug('question_df.items()')
		# logger.debug(self.question_df.items())

		for k,v in self.question_df.items():
			if(k==self.rel_pattern_agg):
				continue
			else:
				user_question_list.append(str(k)+"="+str(v.to_string(index=False)))
		user_question_clause = ',\n  '.join(user_question_list)
		# logger.debug("user_question_list")
		# logger.debug(user_question_clause)

		predict = '' 
		if(len(self.rel_pattern_pred_list)>1):
			predict = 'predict'
		else:
			predict = 'predicts'	

		fixed_pair_list=[]
		rel_pattern_part_value_list = self.rel_pattern_part_value.split(",")

		for n in range(len(self.rel_pattern_part_list)):
			eq = (self.rel_pattern_part_list[n]+"="+rel_pattern_part_value_list[n])
			fixed_pair_list.append(eq)
		if(len(fixed_pair_list)==1):
			fixed_pair = fixed_pair_list[0]
		else:
			fixed_pair = ",".join(fixed_pair_list)

		variable_pair_list=[]
		variable_attr_list = self.rel_pattern_pred_list

		for n in range(len(variable_attr_list)):
			eq = (str(variable_attr_list[n])+"="+str(self.question_df[variable_attr_list[n]].to_string(index=False)))
			variable_pair_list.append(eq)
		if(len(variable_pair_list)==1):
			variable_pair = variable_pair_list[0]
		else:
			variable_pair = ",".join(variable_pair_list)

		counter_dir = ''


		if(user_direction=='high'):
			counter_dir='low'
		else:
			counter_dir='high'


		# logger.debug('counter_dir:')
		# logger.debug(counter_dir)

		exp_tuple_dict = self.explanation_df.to_dict('records')[0]

		exp_list = []
		for k,v in exp_tuple_dict.items():
			if(k==self.rel_pattern_agg or k in self.rel_pattern_part.split(',')):
				continue
			else:
				exp_list.append(str(k)+"="+str(v))
		exp_clause = ','.join(exp_list)

		# logger.debug('exp_clause:')
		# logger.debug(exp_clause)


		# logger.debug('exp_tuple_dict.items()')
		# logger.debug(exp_tuple_dict.items())

		if(self.drill_down_df is None):

			comprehensive_exp = """Explanation for why {} is {}er than expected for: {}. In general, {} {} {}  for most {}. This is also true for {}. However, for {}, {} is {}er than predicted. This may be explained through the {}er than expected outcome for {}.
			""".format(self.rel_pattern_agg,user_direction,user_question_clause,
				str(self.rel_pattern_pred),predict,self.rel_pattern_agg,str(self.rel_pattern_part),fixed_pair,
				variable_pair,self.rel_pattern_agg,user_direction,counter_dir,exp_clause)

			raw_exp = ranking_clause+comprehensive_exp
			raw_exp_lists = textwrap.wrap(raw_exp,width=50)
			final_exp_lists = '\n'.join(raw_exp_lists)

		else:

			drill_pair_list=[]
			for n in range(len(self.drill_attr)):
				eq = (str(self.drill_attr[n])+"="+str(self.explanation_df[self.drill_attr[n]].to_string(index=False)))
				drill_pair_list.append(eq)
			if(len(drill_pair_list)==1):
				drill_pair = drill_pair_list[0]
			else:
				drill_pair = ",".join(drill_pair_list)

			user_question_clause = ','.join(user_question_list)

			comprehensive_exp = """ Explanation for why {} is {}er than expected for: {}.Even though like many other {}, {} {} {} for {}(Left Graph), the fact that  {} is {} can also be explained by \n{}er than usual number of {} in {}(Right Graph).
			""".format(self.rel_pattern_agg,user_direction,user_question_clause,
				str(self.rel_pattern_part),str(self.rel_pattern_pred),predict,self.rel_pattern_agg,
				fixed_pair,user_question_clause,user_direction,
				counter_dir,self.rel_pattern_agg,exp_clause)


			raw_exp = ranking_clause+comprehensive_exp
			raw_exp_lists = textwrap.wrap(raw_exp,width=90)
			final_exp_lists = '\n'.join(raw_exp_lists)

		final_exp_lists = final_exp_lists.replace('name','author')
		final_exp_lists = final_exp_lists.replace('\'','')



		pattern_description = Label(self.exp_frame,text=final_exp_lists,font=('Times New Roman bold',19),bg='white',relief=SOLID,justify=LEFT)
		pattern_description.pack(expand=True)




def main():
	
	relevent_pattern_df = pd.DataFrame({'name':['Boris Glavic','Boris Glavic','Boris Glavic','Boris Glavic'],'venue':['sigmod','vldb','kdd','icde'],'year':['2013','2014','2016','2018'],'pubcount':[5,4,3,10]})

	drill_down_df = pd.DataFrame({'name':['Boris Glavic','Boris Glavic','Boris Glavic','Boris Glavic'],'year':['2013','2014','2016','2018'],'venue':['sigmod','vldb','kdd','icde'],'pubcount':[5,4,1,7]})
	question_df = pd.DataFrame({'name':['Boris Glavic'],'venue':['icde'],'year':['2018'],'pubcount':[10]})
	explanation_df = pd.DataFrame({'name':['Boris Glavic'],'venue':['kdd'],'year':['2016'],'pubcount':[1]})

	exp_chosen_row = pd.DataFrame({'Explanation_Tuple':['Boris Glavic,kdd,2016,3'],'Score':[7.34],
		'From_Pattern':['[name=Boris Glavic]:year \u2933 pubcount'],'Drill_Down_To':['venue'],'Distance':[11.36],
		'Outlierness':[0],'Denominator':[-0.83],'relevent_model':['const'],'relevent_param':[4.5],'refinement_model':['const'],
		'drill_param':[3.2]})

	data_type_dict = {'name': 'str', 'venue': 'str', 'year': 'numeric', 'pubcount': 'numeric'}


	root = Tk()

	exp_frame_1 = Exp_Frame(question_df=question_df, explanation_df=explanation_df, exp_chosen_row=exp_chosen_row, none_drill_down_df=relevent_pattern_df,
		drill_down_df=drill_down_df, data_convert_dict=data_type_dict)

	exp_frame_1.load_exp_graph()

	root.mainloop()
	
if __name__ == '__main__':
	main()




#!/usr/bin/python
# -*- coding:utf-8 -*- 

import sys, getopt
import pandas
import csv
import numpy as np
import time
import math

from capexplain.utils import *


class CategoryNetworkEmbedding(object):
    """Summary of class here.
    """
    def __compute_cos_similarity(self, col, val1, val2):
        if col not in self.cate_cols:
            return -1
        else:
            # if 'vldb' in val1:
            #     val1 = 'vldb'
            # if 'vldb' in val2:
            #     val2 = 'vldb'
            if val1 not in self.vector_list or val2 not in self.vector_list:
                return -1
            list1 = self.vector_list[val1]
            list2 = self.vector_list[val2]
            
            res = 0.0
            len1 = 0.0
            len2 = 0.0
            for i in range(len(list1)):
                res = res + list1[i] * list2[i]
                len1 = len1 + list1[i] * list1[i]
                len2 = len2 + list2[i] * list2[i]
            return res / (math.sqrt(len1) * math.sqrt(len2))
            #return res

    def __init__(self, inf='', df=[], aggr_col='count'):
        """Inits SampleClass with blah."""
        
        self.vector_list = {}
        self.df = df
        self.cate_cols = {}

        if inf == '':
            return
        infile = open(inf, 'r')
        while True:
            line = infile.readline()
            if not line:
                break
            temp_arr = line.strip('\n\t').split(' ')
            self.vector_list[temp_arr[0]] = list(map(lambda x:float(x), temp_arr[1:]))

        for col in df:
            if col == 'index' or col == 'name':
                continue
            if df[col].dtype.kind == 'S' or df[col].dtype.kind == 'O':
                self.cate_cols[col] = True
        venue_list = ['conf/sigmod', 'conf/vldb', 'conf/icde', 'conf/pods', 'journals/pvldb','conf/icml','journals/vldb','conf/kdd','conf/aaai','conf/nips','conf/ijcai']
        for v1 in venue_list:
            for v2 in venue_list:
                print(v1, v2, self.__compute_cos_similarity('pubkey', v1, v2))

    def is_categorical(self, col):
        return col in self.cate_cols

    def compute_similarity(self, col, val1, val2, aggr_col):
        return (1.0 + self.__compute_cos_similarity(col, val1, val2)) * 0.5



try:
    from setuptools.core import setup
except ImportError:
    from distutils.core import setup

from setuptools import find_packages

# Long description is github readme
long_description = None
with open('README.md') as f:
    long_description = f.read()

PACKAGE = 'capexplain'
VERSION = '0.3'

setup(
    name=PACKAGE,
    version=VERSION,
    author='US <e@g.com>',
    author_email='who@gmail.com',
    url='https://github.com/iitdbgroup/cape',
    install_requires=[
        'certifi>=2018.4.16',
        'chardet>=3.0.4',
        'colorful>=0.4.1',
        'geopy',
        'idna>=2.7',
        'matplotlib==3.0.2',
        'numpy>=1.14.5',
        'pandas==0.23.4',
        'patsy>=0.5.0',
        'pkginfo>=1.4.2',
        'psycopg2-binary>=2.7.6',
        'python-dateutil>=2.7.3',
        'pytz>=2018.5',
        'requests>=2.19.1',
        'requests-toolbelt>=0.8.0',
        'scikit-learn>=0.19.2',
        'scipy==1.2.1',
        'six>=1.11.0',
        'sklearn>=0.0',
        'SQLAlchemy==1.2.10',
        'statsmodels>=0.9.0',
        'tqdm>=4.23.4',
        'urllib3<1.24,>=1.23',
        'pandastable'
    ],

    entry_points={
        'console_scripts': [
            'capexplain=capexplain.cape:main',
            'capegui=capexplain.gui.Cape_GUI:main'
        ]
    },

    description='Cape - a system for explaining outliers in aggregation results through counterbalancing.',
    long_description=long_description,
    long_description_content_type='text/markdown',

    packages=find_packages(),

    keywords='db',
    platforms='any',
    license='Apache2',

    classifiers=[
        'Programming Language :: Python',
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',

        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Developers',

        'Topic :: Scientific/Engineering :: Information Analysis',
    ],
)



import pandas as pd
import psycopg2
import tkinter as tk
from tkinter import *
from tkinter import ttk
from matplotlib.figure import Figure
from matplotlib.backends.backend_tkagg import (
    FigureCanvasTkAgg, NavigationToolbar2Tk)
import matplotlib.pyplot as plt
import numpy as np
from mpl_toolkits.mplot3d import Axes3D
from capexplain.gui.Plotting import Plotter
import logging
import textwrap


logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
formatter = logging.Formatter('%(asctime)s %(levelname)s %(name)s line %(lineno)d: %(message)s')
stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
logger.addHandler(stream_handler)


class Local_Pattern_Frame:

	def __init__(self,chosen_row=None,pattern_data_df=None,agg_alias=None,data_convert_dict=None):

		self.chosen_row = chosen_row
		self.pattern_data_df = pattern_data_df
		self.agg_alias = agg_alias
		self.data_convert_dict = data_convert_dict

		self.pop_up_frame = Toplevel()
		self.pop_up_frame.geometry("%dx%d%+d%+d" % (1200, 800, 250, 125))
		self.pop_up_frame.wm_title("Pattern Detail")

		self.win_frame = Frame(self.pop_up_frame)
		self.win_frame.pack(fill=BOTH,expand=True)
		self.win_frame.columnconfigure(0,weight=1)
		self.win_frame.columnconfigure(1,weight=3)
		self.win_frame.rowconfigure(0,weight=1)


	def load_pattern_graph(self):

		graph_frame = Frame(self.win_frame)
		graph_frame.grid(column=1,row=0,sticky='nesw')
		self.figure = Figure(figsize=(5,5),dpi=130)
		canvas = FigureCanvasTkAgg(self.figure,graph_frame)
		canvas.get_tk_widget().pack(side=TOP, fill=BOTH, expand=True)
		toolbar = NavigationToolbar2Tk(canvas,graph_frame)
		toolbar.update()
		canvas.get_tk_widget().pack(side=TOP, fill=BOTH, expand=1)

		if(len(self.pattern_data_df)>=50):
			self.text_plotter = Plotter(figure=self.figure,data_convert_dict=self.data_convert_dict,mode='2D')
			self.text_plotter.add_text("Cannot plot because the size of the data is so large!")

		elif(len(self.chosen_row['variable'])>2):
			self.text_plotter = Plotter(figure=self.figure,data_convert_dict=self.data_convert_dict,mode='2D')
			self.text_plotter.add_text("Cannot plot because the number of dimension of data is higher than 2!")

		elif(self.chosen_row['model']=='const'):
			if(len(self.chosen_row['variable'])==1):
				self.plotter = Plotter(figure=self.figure,data_convert_dict=self.data_convert_dict,mode='2D')
				variable_name= self.chosen_row['variable'][0]
				const=round(self.chosen_row['stats'],2)
				self.plotter.plot_2D_const(const,label="Pattern Model")
				draw_df = self.pattern_data_df[[variable_name,self.agg_alias]]
				logger.debug(draw_df)

				self.plotter.plot_2D_scatter(draw_df,x=variable_name,y=self.agg_alias,label=self.agg_alias)
				self.plotter.set_x_label(variable_name)
				self.plotter.set_y_label(self.agg_alias)
				self.plotter.set_title("Pattern Graph")
		
			else:

				self.plotter = Plotter(figure=self.figure,data_convert_dict=self.data_convert_dict,mode='3D')
				x_name = self.chosen_row['variable'][0]
				y_name = self.chosen_row['variable'][1]
				const = self.chosen_row['stats']
				draw_const_df = self.pattern_data_df[[x_name,y_name]]
				draw_scatter_df = self.pattern_data_df[[x_name,y_name,self.agg_alias]]
				self.plotter.plot_3D_const(draw_const_df,x=x_name,y=y_name,z_value=const,label="Pattern Model")
				self.plotter.plot_3D_scatter(draw_scatter_df,x=x_name,y=y_name,z=self.agg_alias,label=self.agg_alias)
				self.plotter.set_x_label(x_name)
				self.plotter.set_y_label(y_name)
				self.plotter.set_z_label(self.agg_alias)
				self.plotter.set_title("Pattern Graph")

		elif(self.chosen_row['model']=='linear'):
			if(len(self.chosen_row['variable'])==1):

				self.plotter = Plotter(figure=self.figure,data_convert_dict=self.data_convert_dict,mode='2D')
				variable_name = self.chosen_row['variable'][0]
				intercept_value = self.chosen_row['param']['Intercept']
				slope_name = list(self.chosen_row['param'])[1]
				slope_value = float(self.chosen_row['param'][slope_name])

				draw_line_df = self.pattern_data_df[[variable_name]]
				draw_scatter_df = self.pattern_data_df[[variable_name,self.agg_alias]]
				self.plotter.plot_2D_linear(draw_line_df,slope=slope_value,intercept=intercept_value,label="Pattern Model")
				self.plotter.plot_2D_scatter(draw_scatter_df,x=variable_name,y=self.agg_alias,label=self.agg_alias)
				self.plotter.set_x_label(variable_name)
				self.plotter.set_y_label(self.agg_alias)
				self.plotter.set_title("Pattern Graph")

		canvas.draw()


	def load_pattern_description(self):

		fixed_attribute = self.chosen_row['fixed']
		fixed_value = self.chosen_row['fixed_value']
		if(len(fixed_attribute)==1):
			fixed_clause=fixed_attribute[0]+' = '+fixed_value[0]
		else:
			pairs = []
			for n in range(len(fixed_attribute)):
				pair = str(fixed_attribute[n])+' = '+str(fixed_value[n])
				pairs.append(pair)
			fixed_clause=',\n'.join(pairs)
		aggregation_function=self.chosen_row['agg']
		modeltype = self.chosen_row['model']
		variable_attribute = self.chosen_row['variable']
		if(len(variable_attribute)==1):
			variable_attribute=variable_attribute[0]
		else:
			variable_attribute=','.join(variable_attribute)
		if(self.chosen_row['model']=='const'):
			pass
			model_str = "\n"
		else:
			Intercept_value = round((self.chosen_row['param']['Intercept']),2)
			slope_name = list(self.chosen_row['param'])[1]
			slope_value = round((self.chosen_row['param'][slope_name]),2)
			model_str = "\nIntercept: "+str(Intercept_value)+',\n '+str(slope_name)+" as Coefficient: "+str(slope_value)
		theta = "The goodness of fit of the model is "+str(round(self.chosen_row['theta'],2))
		local_desc = "For "+fixed_clause+',the '+self.agg_alias +' is '+modeltype+' in '+variable_attribute+'.'
		local_desc = local_desc.replace('const','constant')
		pattern_attr = model_str+theta
		raw_pattern_description = local_desc+pattern_attr
		raw_pattern_description_lists = textwrap.wrap(raw_pattern_description,width=35)
		final_pattern_description = '\n'.join(raw_pattern_description_lists)

		pattern_description = Label(self.win_frame,text=final_pattern_description,font=('Times New Roman bold',18),borderwidth=5,bg='white',relief=SOLID,justify=LEFT)
		pattern_description.grid(column=0,row=0,sticky='nsew')





import sys
import numpy as np
from sklearn import linear_model
import statsmodels.formula.api as smf
from capexplain.utils import *

class LocalRegressionPattern(object):
    """
        The local regression constraint derived with fixed attributes F, 
        variable attributes V, the assigned values f over fixed attributes F
        threshold \epsilon of deciding whether a tuple $t$ is an outlier 

        Attributes:
            F: fixed attributes
            f_value: the assigned values over F
            V: variable attributes
            epsilon: threshold of deciding outliers
            model: regression model from statsmodels library with the formula agg(B) ~ t[V] 
            regr: model fitted on \pi_{V}(Q_f(R))
            regr_skl: model fitted on \pi_{V}(Q_f(R)) using scikit-learn library
    """

    def __init__(self, F=[], f_value=[], V=[], agg_col='count', epsilon=0.1):
        self.fix_attr = F
        self.fix_attr_value = f_value
        self.var_attr = V
        self.agg_col = agg_col
        self.model = None
        self.regr = None
        self.regr_skl = None
        self.epsilon = epsilon

    def train(self, train_data, formula):
        self.model = smf.ols(formula=formula, data=train_data)
        self.regr = self.model.fit()

    def train_sklearn(self, train_X, train_y):
        self.regr_skl = linear_model.LinearRegression()
        self.regr_skl.fit(train_X, train_y)

    def predict(self, v_values):
        return self.regr.predict(v_values)

    def predict_sklearn(self, v_values):
        return self.regr_skl.predict(v_values)

    def same_fixed_attributes(self, t):
        return self.fix_attr_value == get_F_value(self.fix_attr, t)

    def print_fit_summary(self):
        if self.regr is not None:
            print(self.regr.summary())

    # API for passing fitted regression models
    def set_trained_result_sm(self, regr):
        self.regr = regr

    def set_trained_result_skl(self, regr):
        self.regr_skl = regr
    
